@@ -7,6 +7,12 @@ import dns.resolver
 import dns.exception
 import idna  # implements IDNA 2008; Python's codec is only IDNA 2003
 
+# Default values for keyword arguments.
+
+ALLOW_SMTPUTF8 = True
+CHECK_DELIVERABILITY = True
+TEST_ENVIRONMENT = False
+DEFAULT_TIMEOUT = 15  # secs
 
 # Based on RFC 2822 section 3.2.4 / RFC 5322 section 3.2.3, these
 # characters are permitted in email addresses (not taking into
@@ -20,8 +26,8 @@ DOT_ATOM_TEXT = '[' + ATEXT + ']+(?:\\.[' + ATEXT + ']+)*'
 # addresses to also include three specific ranges of UTF8 defined in
 # RFC3629 section 4, which appear to be the Unicode code points from
 # U+0080 to U+10FFFF.
-ATEXT_UTF8 = ATEXT + u"\u0080-\U0010FFFF"
-DOT_ATOM_TEXT_UTF8 = '[' + ATEXT_UTF8 + ']+(?:\\.[' + ATEXT_UTF8 + ']+)*'
+ATEXT_INTL = ATEXT + u"\u0080-\U0010FFFF"
+DOT_ATOM_TEXT_INTL = '[' + ATEXT_INTL + ']+(?:\\.[' + ATEXT_INTL + ']+)*'
 
 # The domain part of the email address, after IDNA (ASCII) encoding,
 # must also satisfy the requirements of RFC 952/RFC 1123 which restrict
@@ -39,21 +45,67 @@ DOMAIN_MAX_LENGTH = 255
 # IANA Special Use Domain Names
 # Last Updated 2021-09-21
 # https://www.iana.org/assignments/special-use-domain-names/special-use-domain-names.txt
+#
 # The domain names without dots would be caught by the check that the domain
 # name in an email address must have a period, but this list will also catch
 # subdomains of these domains, which are also reserved.
-SPECIAL_USE_DOMAIN_NAMES = (
-    "arpa",  # consolidated from a lot of arpa subdomains, we'll assume all subdomains of arpa are actually reserved
-    "example",
-    "example.com",
-    "example.net",
-    "example.org",
+SPECIAL_USE_DOMAIN_NAMES = [
+    # The "arpa" entry here is consolidated from a lot of arpa subdomains
+    # for private address (i.e. non-routable IP addresses like 172.16.x.x)
+    # reverse mapping, plus some other subdomains. Although RFC 6761 says
+    # that application software should not treat these domains as special,
+    # they are private-use domains and so cannot have globally deliverable
+    # email addresses, which is an assumption of this library, and probably
+    # all of arpa is similarly special-use, so we reject it all.
+    "arpa",
+
+    # RFC 6761 says applications "SHOULD NOT" treat the "example" domains
+    # as special, i.e. applications should accept these domains.
+    #
+    # The domain "example" alone fails our syntax validation because it
+    # lacks a dot (we assume no one has an email address on a TLD directly).
+    # "@example.com/net/org" will currently fail DNS-based deliverability
+    # checks because IANA publishes a NULL MX for these domains, and
+    # "@mail.example[.com/net/org]" and other subdomains will fail DNS-
+    # based deliverability checks because IANA does not publish MX or A
+    # DNS records for these subdomains.
+    # "example", # i.e. "wwww.example"
+    # "example.com",
+    # "example.net",
+    # "example.org",
+
+    # RFC 6761 says that applications are permitted to treat this domain
+    # as special and that DNS should return an immediate negative response,
+    # so we also immediately reject this domain, which also follows the
+    # purpose of the domain.
     "invalid",
+
+    # RFC 6762 says that applications "may" treat ".local" as special and
+    # that "name resolution APIs and libraries SHOULD recognize these names
+    # as special," and since ".local" has no global definition, we reject
+    # it, as we expect email addresses to be gloally routable.
     "local",
+
+    # RFC 6761 says that applications (like this library) are permitted
+    # to treat "localhost" as special, and since it cannot have a globally
+    # deliverable email address, we reject it.
     "localhost",
+
+    # RFC 7686 says "applications that do not implement the Tor protocol
+    # SHOULD generate an error upon the use of .onion and SHOULD NOT
+    # perform a DNS lookup.
     "onion",
-    "test",  # see special logic for 'test' where this is checked
-)
+
+    # Although RFC 6761 says that application software should not treat
+    # these domains as special, it also warns users that the address may
+    # resolve differently in different systems, and therefore it cannot
+    # have a globally routable email address, which is an assumption of
+    # this library, so we reject "@test" and "@*.test" addresses, unless
+    # the test_environment keyword argument is given, to allow their use
+    # in application-level test environments. These domains will generally
+    # fail deliverability checks because "test" is not an actual TLD.
+    "test",
+]
 
 # ease compatibility in type checking
 if sys.version_info >= (3,):
@@ -66,8 +118,6 @@ else:
     DOT_ATOM_TEXT = DOT_ATOM_TEXT.decode("ascii")
     ATEXT_HOSTNAME = ATEXT_HOSTNAME.decode("ascii")
 
-DEFAULT_TIMEOUT = 15  # secs
-
 
 class EmailNotValidError(ValueError):
     """Parent class of all exceptions raised by this module."""
@@ -210,10 +260,10 @@ def caching_resolver(timeout=DEFAULT_TIMEOUT, cache=None):
 
 def validate_email(
     email,
-    allow_smtputf8=True,
+    allow_smtputf8=ALLOW_SMTPUTF8,
     allow_empty_local=False,
-    check_deliverability=True,
-    test_environment=False,
+    check_deliverability=CHECK_DELIVERABILITY,
+    test_environment=TEST_ENVIRONMENT,
     timeout=DEFAULT_TIMEOUT,
     dns_resolver=None
 ):
@@ -350,11 +400,11 @@ def validate_email_local_part(local, allow_smtputf8=True, allow_empty_local=Fals
 
     else:
         # The local part failed the ASCII check. Now try the extended internationalized requirements.
-        m = re.match(DOT_ATOM_TEXT_UTF8 + "\\Z", local)
+        m = re.match(DOT_ATOM_TEXT_INTL + "\\Z", local)
         if not m:
             # It's not a valid internationalized address either. Report which characters were not valid.
             bad_chars = ', '.join(sorted(set(
-                unicodedata.name(c, repr(c)) for c in local if not re.match(u"[" + (ATEXT if not allow_smtputf8 else ATEXT_UTF8) + u"]", c)
+                unicodedata.name(c, repr(c)) for c in local if not re.match(u"[" + (ATEXT if not allow_smtputf8 else ATEXT_INTL) + u"]", c)
             )))
             raise EmailSyntaxError("The email address contains invalid characters before the @-sign: %s." % bad_chars)
 
@@ -370,7 +420,7 @@ def validate_email_local_part(local, allow_smtputf8=True, allow_empty_local=Fals
 
         # Check for unsafe characters.
         # Some of this may be redundant with the range U+0080 to U+10FFFF that is checked
-        # by DOT_ATOM_TEXT_UTF8.
+        # by DOT_ATOM_TEXT_INTL.
         for i, c in enumerate(local):
             category = unicodedata.category(c)
             if category[0] in ("L", "N", "P", "S"):
@@ -501,11 +551,7 @@ def validate_email_domain_part(domain, test_environment=False):
     # Some might fail DNS-based deliverability checks, but that
     # can be turned off, so we should fail them all sooner.
     for d in SPECIAL_USE_DOMAIN_NAMES:
-        # RFC 6761 says that applications should not block use of the 'test'
-        # domain name, presumably because that would prevent it from being
-        # used for actual testing. We'll block it, except when a special
-        # testing flag is used, indicating that the module is being used
-        # in a test environment.
+        # See the note near the definition of SPECIAL_USE_DOMAIN_NAMES.
         if d == "test" and test_environment:
             continue
 
